package org.xbib.datastructures.multi;

import java.util.Iterator;
import java.util.Map;
import java.util.Objects;
import java.util.function.Consumer;

public class MapsKeySet<K extends Object, V extends Object> extends ImprovedAbstractSet<K> {
    final Map<K, V> map;

    MapsKeySet(Map<K, V> map) {
        this.map = Objects.requireNonNull(map);
    }

    Map<K, V> map() {
        return map;
    }

    @Override
    public Iterator<K> iterator() {
        return keyIterator(map().entrySet().iterator());
    }

    @Override
    public void forEach(Consumer<? super K> action) {
        Objects.requireNonNull(action);
        // avoids entry allocation for those maps that allocate entries on iteration
        map.forEach((k, v) -> action.accept(k));
    }

    @Override
    public int size() {
        return map().size();
    }

    @Override
    public boolean isEmpty() {
        return map().isEmpty();
    }

    @Override
    public boolean contains(Object o) {
        return map().containsKey(o);
    }

    @Override
    public boolean remove(Object o) {
        if (contains(o)) {
            map().remove(o);
            return true;
        }
        return false;
    }

    @Override
    public void clear() {
        map().clear();
    }

    private static <K extends Object, V extends Object> Iterator<K> keyIterator(
            Iterator<Map.Entry<K, V>> entryIterator) {
        return new TransformedIterator<Map.Entry<K, V>, K>(entryIterator) {
            @Override
            K transform(Map.Entry<K, V> entry) {
                return entry.getKey();
            }
        };
    }
}
