package org.xbib.datastructures.multi;

import java.util.Objects;
import org.xbib.datastructures.api.Multiset;

/**
 * Implementation of the {@code equals}, {@code hashCode}, and {@code toString} methods of {@link
 * Multiset.Entry}.
 */
public abstract class MultisetsAbstractEntry<E extends Object> implements Multiset.Entry<E> {
    /**
     * Indicates whether an object equals this entry, following the behavior specified in {@link
     * Multiset.Entry#equals}.
     */
    @Override
    public boolean equals(Object object) {
        if (object instanceof Multiset.Entry) {
            Multiset.Entry<?> that = (Multiset.Entry<?>) object;
            return this.getCount() == that.getCount()
                    && Objects.equals(this.getElement(), that.getElement());
        }
        return false;
    }

    /**
     * Return this entry's hash code, following the behavior specified in {@link
     * Multiset.Entry#hashCode}.
     */
    @Override
    public int hashCode() {
        E e = getElement();
        return ((e == null) ? 0 : e.hashCode()) ^ getCount();
    }

    /**
     * Returns a string representation of this multiset entry. The string representation consists of
     * the associated element if the associated count is one, and otherwise the associated element
     * followed by the characters " x " (space, x and space) followed by the count. Elements and
     * counts are converted to strings as by {@code String.valueOf}.
     */
    @Override
    public String toString() {
        String text = String.valueOf(getElement());
        int n = getCount();
        return (n == 1) ? text : (text + " x " + n);
    }
}
