package org.xbib.datastructures.multi;

import java.util.Iterator;
import java.util.Objects;

/**
 * An iterator that transforms a backing iterator; for internal use.
 */
abstract class TransformedIterator<F extends Object, T extends Object>
        implements Iterator<T> {
    final Iterator<? extends F> backingIterator;

    TransformedIterator(Iterator<? extends F> backingIterator) {
        this.backingIterator = Objects.requireNonNull(backingIterator);
    }

    abstract T transform(F from);

    @Override
    public final boolean hasNext() {
        return backingIterator.hasNext();
    }

    @Override
    public final T next() {
        return transform(backingIterator.next());
    }

    @Override
    public final void remove() {
        backingIterator.remove();
    }
}
